"""Reports by email
"""

from marrow.mailer import Message, Mailer


def send_report_mail(plain_text, settings):
    """Send anomalies report by email

    Arguments:
        plain_text {string} -- Report body in plain text
        settings {object} -- Settings from the class Config

    Returns:
    """

    mailer = Mailer({
        'manager.use': 'futures',
        'transport.use': 'smtp',
        'transport.host': settings.smtp_host,
        'transport.tls': settings.smtp_ssl,
        'transport.debug': False,
        'transport.username': settings.smtp_user,
        'transport.password': settings.smtp_pass,
        'transport.max_messages_per_connection': 5
    })

    mailer.start()

    message = Message(author=settings.smtp_from, to=settings.smtp_to)
    message.subject = settings.smtp_subject
    message.plain = plain_text

    mailer.send(message)
    mailer.stop()

    return True



"""IP geolocation from ipinfo.io service.

https://ipinfo.io
"""

import requests


def check_geoip(ip_address, token):
    """check_geoip

    Arguments:
        ip_address {string} -- IP address to be checked
        token {string}      -- ipinfo token

    Returns:
        {json} -- IP address information returned from ipinfo.io
    """

    url = 'http://ipinfo.io/' + ip_address + '?token=' + token

    try:
        response = requests.get(url)
    except requests.RequestException:
        return False

    return response.json()



"""Processor postfix_sasl
"""

from collections import defaultdict
import logging
import ipaddress
import re

from loganom import utils, process_mail, report_mail, report_mm, exec_cmd


def postfix_sasl(settings, args):
    """postfix_sasl

    Arguments:
        settings {object} -- Settings from the class Config
        args {object} -- Execution parameters, where is the log path
    """

    pattern_sasl = 'sasl_method='

    logging.debug('Log file: %s', args.log.name)
    logfile_path = args.log.name

    # Initialize empty dictionary
    dict_general = defaultdict(set)

    with open(logfile_path) as log_file:
        logging.debug('Start log reading...')

        for line in log_file:

            if pattern_sasl in line:
                lista = line.split()

                mail_user = lista.pop().split('sasl_username=')[-1]

                # >/dev/null, ignore this
                lista.pop()

                mail_ip_address = utils.clean_ip(lista.pop())

                logging.debug('%s - %s', mail_user, mail_ip_address)

                if ipaddress.ip_address(mail_ip_address).is_private:
                    logging.debug('%s private ip address, skipped', mail_ip_address)
                else:
                    dict_general[mail_user].add(mail_ip_address)

    logging.debug('End log reading...')
    temp_set = utils.process_dict(dict_general)
    temp_dict = process_mail.process_mail(dict_general, temp_set, settings)

    # Results
    report_text = ""
    if len(temp_dict) > 0:
        for email in temp_dict.keys():

            # Execute external script when an anomaly is found for each e-mail
            if args.exec:
                exec_cmd.external_exec(args.exec.name, email)

            report_text += "\nE-mail address: {}\n".format(email)
            for ip_auth in temp_dict[email]:
                report_text += "\t{}\n".format(ip_auth)

        # Report in screen
        print(report_text)

        if settings.smtp_enabled:
            report_mail.send_report_mail(report_text, settings)

        if settings.mm_enabled:
            report_mm.send_report_mm(report_text, settings)
    else:
        logging.debug('No anomalies')



def quota_high(settings, args):
    """quota_high

    Arguments:
        settings {object} -- Settings from the class Config
        args {object} -- Execution parameters, where is the log path
    """

    pattern = args.quota_message

    logging.debug('Log file: %s', args.log.name)
    logfile_path = args.log.name

    # Initialize empty dictionary
    dict_general = defaultdict(int)

    with open(logfile_path) as log_file:
        logging.debug('Start log reading...')

        for line in log_file:
            if pattern in line:
                mail_user_raw = re.search('from=<.+?>', line)

                mail_user = utils.clean_email(mail_user_raw.group(0))

                dict_general[mail_user] += 1

    logging.debug('End log reading...')

    # Results
    email_quota_count = 0
    report_text = "Email address(es) that hit quota limit (> {}):\n".format(args.quota_limit)

    for email, quota_counter in dict_general.items():

        logging.debug('%s (%i)', email, quota_counter)

        if quota_counter > int(args.quota_limit):
            report_text += "  {} ({})\n".format(email, quota_counter)
            email_quota_count += 1

            # Execute external script when an anomaly is found for each e-mail
            if args.exec:
                exec_cmd.external_exec(args.exec.name, email)

    if email_quota_count > 0:
        # Report in screen
        print(report_text)

        if settings.smtp_enabled:
            report_mail.send_report_mail(report_text, settings)

        if settings.mm_enabled:
            report_mm.send_report_mm(report_text, settings)
    else:
        logging.debug('No anomalies')



"""loganom main

Check parameters and user settings then execute the selected processor.
"""

import os
import sys
import argparse
import logging

from loganom import config
from loganom.processor import postfix_sasl, quota_high


logging.basicConfig(
    level=os.environ.get("LOGLEVEL", "INFO"),
    format="%(levelname)s:\t%(message)s")


def main():
    """loganom main function
    """

    parser = argparse.ArgumentParser()

    parser.add_argument('processor',
                        choices=['postfix-sasl', 'quota-high', 'foo'],)

    parser.add_argument('-c', '--config',
                        type=argparse.FileType('r'),
                        help='Path for configuration file (default: ./config.ini)',
                        default='./config.ini')

    parser.add_argument('-l', '--log',
                        type=argparse.FileType('r'),
                        help='Path for log file (default: /var/log/maillog)',
                        default='/var/log/maillog')

    parser.add_argument('-e', '--exec',
                        type=argparse.FileType('r'),
                        help='External script to be executed when an anomaly is found',
                        required=False)

    parser.add_argument('-q', '--quota-message',
                        help='''
                        Quota reject message used in the mail server
                        (default: 'Quota per hour exceeded')
                        [Processor quota-high]''',
                        default='Quota per hour exceeded',
                        required=False)

    parser.add_argument('--quota-limit',
                        help='''
                        Quota limit occurrences, above this it will be considered
                        an anomaly (default: 150)
                        [Processor quota-high]''',
                        default=150,
                        required=False)

    args = parser.parse_args()

    settings = config.read_config(args.config)

    if args.processor == 'foo':
        print('[bar]')
        logging.debug('Starting "bar"')
        sys.exit(0)
    elif args.processor == 'quota-high':
        print('[quota-high]')
        logging.debug('Starting "quota-high"')
        quota_high(settings, args)
        sys.exit(0)
    elif args.processor == 'postfix-sasl':
        print('[postfix-sasl]')
        logging.debug('Starting "postfix-sasl"')
        postfix_sasl(settings, args)
        sys.exit(0)


if __name__ == "__main__":
    main()
